package com.camunda.hemera.connect;

import java.io.IOException;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.TimeUnit;

import javax.jcr.LoginException;
import javax.jcr.Node;
import javax.jcr.NodeIterator;
import javax.jcr.PropertyIterator;
import javax.jcr.RepositoryException;
import javax.jcr.Session;
import javax.jcr.SimpleCredentials;
import javax.jcr.Value;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import org.jboss.security.config.IDTrustConfiguration;
import org.modeshape.common.collection.Problem;
import org.modeshape.connector.svn.SvnRepositorySource;
import org.modeshape.graph.ExecutionContext;
import org.modeshape.graph.Graph;
import org.modeshape.graph.JcrLexicon;
import org.modeshape.graph.Location;
import org.modeshape.graph.Subgraph;
import org.modeshape.graph.connector.RepositoryConnection;
import org.modeshape.graph.connector.RepositoryConnectionFactory;
import org.modeshape.graph.connector.RepositoryContext;
import org.modeshape.graph.connector.RepositorySourceException;
import org.modeshape.graph.observe.Observer;
import org.modeshape.graph.property.Name;
import org.modeshape.graph.property.Property;
import org.modeshape.jcr.JcrConfiguration;
import org.modeshape.jcr.JcrEngine;
import org.modeshape.repository.ModeShapeConfiguration;
import org.modeshape.repository.ModeShapeConfiguration.RepositorySourceDefinition;
import org.xml.sax.SAXException;
import org.jboss.security.config.IDTrustConfiguration;
import org.jboss.security.config.IDTrustConfiguration;

/**
 * 
 */
public class SvnViaModeShapeTest extends TestCase {
    
	private ExecutionContext context;
	private SvnRepositorySource source;

	public SvnViaModeShapeTest(String testName ) {
        super( testName );
    }

    public static Test suite() {
        return new TestSuite( SvnViaModeShapeTest.class );
    }

    public void testModeShapeAPI() throws RepositoryException {
        String repositoryUrl = "xxx";
        String[] predefinedWorkspaceNames = new String[] {"trunk"};
        context = new ExecutionContext();
        source = new SvnRepositorySource();
        source.setName("svn repository source");
        source.setRepositoryRootUrl(repositoryUrl);
        source.setUsername("anonymous");
        source.setPassword("");
        source.setCreatingWorkspacesAllowed(true);
        source.setPredefinedWorkspaceNames(predefinedWorkspaceNames);
        source.setDefaultWorkspaceName(predefinedWorkspaceNames[0]);
        source.setCreatingWorkspacesAllowed(false);
        source.initialize(new RepositoryContext() {
            public Subgraph getConfiguration( int depth ) {
                return null;
            }
            public ExecutionContext getExecutionContext() {
                return context;
            }
            public Observer getObserver() {
                return null;
            }
            public RepositoryConnectionFactory getRepositoryConnectionFactory() {
                return new RepositoryConnectionFactory() {
                    public RepositoryConnection createConnection( String sourceName ) throws RepositorySourceException {
                        return null;
                    }
                };
            }
        });
        
        Graph graph = Graph.create(source, context);
//        Map<Name, Property> properties = graph.getPropertiesByName().on("/");

        org.modeshape.graph.Node root = graph.getNodeAt("/");
//        assertThat(root, is(notNullValue()));
//        assertThat(root.getLocation(), is(notNullValue()));
//        assertThat(root.getChildren().isEmpty(), is(false));
        System.out.println(root);
        for (Location childLocation : root.getChildren()) {
//            assertThat(childLocation.getPath().getParent().isRoot(), is(true));
//            // Node child = graph.getNodeAt(childLocation);
//            // assertThat(child.getLocation(), is(childLocation));
//            // assertThat(child.getLocation().getPath().getParent().isRoot(), is(true));
            System.out.println("   " + childLocation);
        }
    }
    
	private void tearDown(Session session, JcrEngine engine) throws InterruptedException {
		session.logout();
		engine.shutdown();
    	engine.awaitTermination(3, TimeUnit.SECONDS);
        System.out.println("## SHUTDOWN COMPLETED ##");
	}

	private Session setupJcrSession(JcrEngine engine)
			throws RepositoryException, LoginException {
		javax.jcr.Repository repository = engine.getRepository("development svn");

        System.out.println("## REPOSITORY CREATED ##");

    	Session session = repository.login(new SimpleCredentials("bernd", "xx".toCharArray()));

        System.out.println("## LOGGED IN AND CREATED SESSION ##");
		return session;
	}

	private JcrEngine setupJcrEngine() throws IOException, SAXException {
		JcrConfiguration config = new JcrConfiguration();
    	config.loadFrom( this.getClass().getResourceAsStream("/svn-config.xml") );
    	JcrEngine engine = config.build();
    	engine.start();

        System.out.println("## STARTED JCR ENGINE ##");

        if (engine.getProblems().hasProblems()) {
            for (Problem problem : engine.getProblems()) {
                System.err.println(problem.getMessageString());
            }
            throw new RuntimeException("Could not start due to problems");
        }

        System.out.println("## NO PROBLEM OCCURED WHILE STARTING JCR ENGINE ##");
		return engine;
	}

	private void setupJaas() {
		// Set up the JAAS provider (IDTrust) and a policy file (which defines the "modeshape-jcr" login config name)
        IDTrustConfiguration idtrustConfig = new IDTrustConfiguration();
        try {
            idtrustConfig.config("security/jaas.conf.xml");
        } catch (Exception ex) {
            throw new IllegalStateException(ex);
        }
        System.out.println("## STARTED JAAS ##");
	}

	private void print(Node node, String intend) throws RepositoryException {
//		System.out.println(intend + node.getName() + "   " + node.getDefinition());
//		System.out.println(node.getProperties().nextProperty().getName());
		if (node.hasProperty("jcr:primaryType")) {
			try {
				Value v = node.getProperty("jcr:primaryType").getValue(); //JcrLexicon.PRIMARY_TYPE.getNamespaceUri());
				if ("nt:file".equals(v.getString())) {
		//			node.getNode("jcr:content");
					if (node.getNode("jcr:content").hasProperty("jcr:data")) {
						System.out.println(intend + "FILE -> " + node.getName() + " -> " ); 
		//						+ node.getNode("jcr:content").getProperty("jcr:data").getValue().getString());
					}
					else {
						System.out.println(intend + "FILE -> " + node.getName() + "  IS EMPTY");
					}
				}
				if ("nt:folder".equals(v.getString())) {
					System.out.println(intend + "FOLDER -> " + node.getName());
				}
			}
			catch (Exception ex) {
				System.out.println("ERROR ON: " + node.getName());
				PropertyIterator iter = node.getProperties();
				while (iter.hasNext()) {
					System.out.println("         property:" + iter.nextProperty().getName());
				}
				ex.printStackTrace();
			}
		}
		else {
			System.out.println("NO PRIMARY TYPE: " + node.getName());
		}
		NodeIterator iter = node.getNodes();
		while (iter.hasNext()) {
			Node nextNode = iter.nextNode();
			print(nextNode, intend + "   ");
		}		
	}
    
    public void testBasicSvnAccess() throws IOException, SAXException, RepositoryException, InterruptedException {
        setupJaas();    	
    	JcrEngine engine = setupJcrEngine();
    	Session session = setupJcrSession(engine);

    	print(session.getRootNode(), "");

    	tearDown(session, engine);
    	engine.shutdown();
    }
    
	public void testReadDirectoryTree() throws Exception {
        setupJaas();    	
    	JcrEngine engine = setupJcrEngine();
    	Session session = setupJcrSession(engine);

    	Node node = session.getRootNode();
	

    	tearDown(session, engine);
    	engine.shutdown();		
	}

	public void testReadFile() throws Exception {
		
	}

	public void testAddFile() throws Exception {
		
	}

	public void testChangeFile() throws Exception {
		
	}

	public void testDeleteFile() throws Exception {
		
	}

	private void testAddDirectory() throws Exception {
		
	}

}
